#!/usr/bin/env python
import argparse
import os
from pathlib import Path

from PIL import Image

from wim.image import IMAGE_FORMATS, add_image, add_text, set_background


def main(args=None) -> None:
    parser = argparse.ArgumentParser(description='Add text and manipulate images.')

    parser.add_argument(
        'filename', type=str, nargs='+', help='Input image filename. Use wildcard to process multiple files.'
    )
    parser.add_argument('--font', default='arial.ttf', help='Set the font family, default is arial.ttf.')
    parser.add_argument('--font-size', type=int, default=16, help='Set the font size, default is 16.')
    parser.add_argument(
        '--format',
        default='webp',
        choices=IMAGE_FORMATS,
        help='Output format (overrides input format)',
    )
    parser.add_argument(
        '-q', '--quantize', action='store_true', help='Quantize the image to reduce its filesize, default is False.'
    )
    parser.add_argument(
        '-s',
        '--scale',
        type=int,
        nargs=2,
        metavar=('WIDTH', 'HEIGHT'),
        help='Set the maximum width and height as integer values.',
    )
    parser.add_argument('-t', '--text', help='Set the text to append at the bottom of the image.')
    parser.add_argument('-w', '--watermark', help='Path to watermark/overlay image to add to the image.')
    parser.add_argument(
        '--watermark-position',
        default='bottom-right',
        choices=['top-left', 'top-right', 'bottom-left', 'bottom-right', 'center'],
        help='Position of watermark (default: bottom-right).',
    )
    parser.add_argument('--watermark-opacity', type=int, default=255, help='Opacity of watermark 0-255 (default: 255).')
    parser.add_argument(
        '--watermark-scale',
        type=int,
        nargs=2,
        metavar=('WIDTH', 'HEIGHT'),
        help='Scale watermark to WIDTH HEIGHT in pixels.',
    )

    output_group = parser.add_mutually_exclusive_group()
    output_group.add_argument(
        '-i', '--inplace', action='store_true', help='Edit the image in place (overwrites original).'
    )
    output_group.add_argument('-o', '--outdir', help='Output directory for processed images.')

    argv = parser.parse_args(args)

    if argv.inplace and argv.format:
        parser.error('--format cannot be used with --inplace (inplace preserves original format)')

    for name in argv.filename:
        img = Image.open(name)
        p_src = Path(name)

        # Set destination path
        parent = Path(argv.outdir) if argv.outdir else p_src.parent
        parent.mkdir(exist_ok=True)
        p_dst = p_src if argv.inplace else parent / f'{p_src.stem}-wim.{argv.format}'

        if argv.quantize:
            img = img.quantize()  # type: ignore

        if argv.scale:
            img.thumbnail(argv.scale)

        if argv.text:
            img = add_text(img, argv.font, argv.font_size, argv.text)

        if argv.watermark:
            img = add_image(
                img,
                argv.watermark,
                position=argv.watermark_position,
                scale=argv.watermark_scale,
                opacity=argv.watermark_opacity,
            )

        # Convert RGBA to RGB for JPEG files
        if p_dst.suffix.lower() in ['.jpg', '.jpeg'] and img.mode == 'RGBA':
            img = set_background(img)

        # Keep timestamps of original image. The stat call must take place before save because of possible inplace edits.
        stat = p_src.stat()
        img.save(p_dst)
        os.utime(p_dst, (stat.st_atime, stat.st_mtime))


if __name__ == '__main__':
    main()



from PIL import Image, ImageDraw, ImageFont

IMAGE_FORMATS = ['bmp', 'jpeg', 'jpg', 'png', 'webp']
MODE = 'RGBA'
BLACK = (0, 0, 0, 255)
WHITE = (255, 255, 255, 255)
FULL_OPACITY = 255


def calculate_position(base_size, overlay_size, position, padding):
    """
    Calculate the position for placing an overlay on a base image.

    Args:
        base_size: Tuple of (width, height) for the base image
        overlay_size: Tuple of (width, height) for the overlay
        position: One of 'top-left', 'top-right', 'bottom-left', 'bottom-right', 'center'
        padding: Padding in pixels from the edges

    Returns:
        Tuple of (x, y) coordinates for the overlay position
    """
    base_width, base_height = base_size
    overlay_width, overlay_height = overlay_size

    positions = {
        'top-left': (padding, padding),
        'top-right': (base_width - overlay_width - padding, padding),
        'bottom-left': (padding, base_height - overlay_height - padding),
        'bottom-right': (base_width - overlay_width - padding, base_height - overlay_height - padding),
        'center': ((base_width - overlay_width) // 2, (base_height - overlay_height) // 2),
    }

    return positions.get(position, positions['bottom-right'])


def ensure_rgba(img):
    """Convert to RGBA if needed."""

    return img if img.mode == MODE else img.convert(MODE)


def add_text(img, font, font_size, text, bg_alpha=32, position='bottom-right', padding=0):
    """
    Add text with a semi-transparent background to an image.

    Args:
        img: PIL Image object
        font: Path to font file
        font_size: Size of the font
        text: Text to add
        bg_alpha: Alpha value for background (0-255, default 32)
        position: Position of text ('top-left', 'top-right', 'bottom-left', 'bottom-right', 'center')
        padding: Padding from edges in pixels

    Returns:
        PIL Image object with text added
    """

    font = ImageFont.truetype(font, font_size)
    bbox = font.getbbox(text)
    text_width = bbox[2] - bbox[0]
    text_height = bbox[3] - bbox[1]
    text_img_height = int(text_height * 1.5)
    text_img_width = int(text_width * 1.1)

    # Create a transparent overlay the same size as the base image
    base_overlay = Image.new(MODE, img.size, (0, 0, 0, 0))
    draw = ImageDraw.Draw(base_overlay, MODE)

    # Calculate position
    x_pos, y_pos = calculate_position(img.size, (text_img_width, text_img_height), position, padding)

    # Draw semi-transparent background rectangle
    bg_color = (0, 0, 0, bg_alpha)
    draw.rectangle([(x_pos, y_pos), (x_pos + text_img_width, y_pos + text_img_height)], fill=bg_color)

    # Composite the semi-transparent background first
    base_layer = Image.alpha_composite(ensure_rgba(img), base_overlay)

    # Now draw the fully opaque text on a new overlay
    text_overlay = Image.new(MODE, img.size, (0, 0, 0, 0))
    text_draw = ImageDraw.Draw(text_overlay, MODE)

    # Calculate centered text position
    text_x = x_pos + (text_img_width - text_width) / 2
    text_y = y_pos + (text_img_height - text_height) / 2

    # Draw text with full opacity (alpha = 255)
    text_draw.text((text_x, text_y), text, WHITE, font=font)

    # Composite the text overlay
    return Image.alpha_composite(base_layer, text_overlay)


def add_image(img, overlay_path, position='bottom-right', padding=0, scale=None, opacity=FULL_OPACITY):
    """
    Blend an image over the base image.

    Args:
        img: PIL Image object (base image)
        overlay_path: Path to the overlay image file
        position: Position of overlay ('top-left', 'top-right', 'bottom-left', 'bottom-right', 'center')
        padding: Padding from edges in pixels
        scale: Scale overlay to this size (width, height) or None to keep original size
        opacity: Opacity of the overlay (0-255, default 255 for fully opaque)

    Returns:
        PIL Image object with overlay added
    """

    # Load and prepare overlay image
    overlay_img = ensure_rgba(Image.open(overlay_path))

    # Scale if requested
    if scale:
        overlay_img.thumbnail(scale)

    # Adjust opacity if needed
    if opacity < FULL_OPACITY:
        # Create a copy to avoid modifying the original
        overlay_img = overlay_img.copy()
        # Get alpha channel and adjust it
        alpha = overlay_img.getchannel('A')
        alpha = alpha.point(lambda x: int(x * opacity / FULL_OPACITY))
        overlay_img.putalpha(alpha)

    # Create a transparent canvas the same size as the base image
    canvas = Image.new(MODE, img.size, (0, 0, 0, 0))

    # Calculate position
    x_pos, y_pos = calculate_position(img.size, overlay_img.size, position, padding)

    # Paste overlay onto canvas
    canvas.paste(overlay_img, (x_pos, y_pos), overlay_img)

    # Composite with base image
    return Image.alpha_composite(ensure_rgba(img), canvas)


def set_background(img):
    """
    Convert RGBA image to RGB with black background.

    Args:
        img: PIL Image object in RGBA mode

    Returns:
        PIL Image object in RGB mode
    """
    rgb_img = Image.new('RGB', img.size, BLACK)
    rgb_img.paste(img, mask=img.split()[3])  # Use alpha channel as mask
    return rgb_img
